/* $Id: histogram.c,v 1.4 2015/10/05 05:11:12 leavens Exp leavens $ */
/* histogram -- print histogram of raw scores
 * SYNOPSIS: User inputs the scores, one per line on standard input.
 * A histogram is printed on standard output.
 */
#include <stdio.h>
#include <string.h>
#include "prompt.h"
#include "print_errors.h"
#define MAX_SCORE 200
#define HSIZE ((MAX_SCORE)+1)

int readnum(int *temp) 
  /* effect: read a number from standard input and place it into temp
     ensures: result is 1 if a number was successfully read into temp, 
              and result is 0 otherwise.
  */
{
  char linebuf[80];
  strcpy(linebuf, "\n");  /* initialization */
  gets(linebuf);
  return sscanf(linebuf, "%d", temp);
}
int main(int argc, char *argv[])
{
  int n, i, temp;
  int lowi = HSIZE;   /* index of low score */
  int highi = 0;    /* index of high score */
  int hgram[HSIZE];   /* scores for each student */

  program_usage_init(argv[0], "");
  argc--;
  if (argc >= 1) {
    usage();
  }
  /* initialize hgram */
  for (n = 0; n < HSIZE; n++) {
    hgram[n] = 0;
  }
  /* get scores from standard input */
  prompt(stdin, "scores (1 per line, stop with EOF (Ctrl-D))?\n");
  for(; (readnum(&temp) == 1);) {
    if (0 <= temp) {
      hgram[temp] += 1;
      if (temp < lowi) { lowi = temp; }
      if (temp > highi) { highi = temp; }
    } else {
      fprintf(stderr, "Problem storing into hgram array (at address %p)\n",
	      (void *)hgram);
      error("illegal score: %d ", temp);
    }
  }
  /* print histogram, starting with the highest value */
  for (n = highi; n >= lowi; n--) {
    printf("%3d: ", n);
    for (i = 0; i < hgram[n]; i++) {
      printf("*");
    }
    printf("\n");
  }
  return 0;
}
