/* $Id: print_errors.c,v 1.3 2015/10/05 05:06:11 leavens Exp leavens $ */
/* print_errors -- routines to print error messages with name of the program
 * SYNOPSIS: All output is to standard error
 *     program_usage_init(argv[0], "options arguments");
 *     usage();
 *     error("you blew it");
 *     warning("This is your %drd warning", 3);
 *     sys_err("can't open file %s", f);
 *     sys_warning("can't open file %s, continuing", f);
 */
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
static char * Name = NULL;
static char * Usage = "USAGE NOT DEFINED";

void program_usage_init(char *name, char *usage_mesg)
     /* effect: initialize the error printing system */
{
  char * lastslash = strrchr(name, '/');
  Name = (lastslash != NULL) ? lastslash+1 : name;
  Usage = (char *) malloc(strlen(usage_mesg));
  strcpy(Usage, usage_mesg);
}
static void print_name(void)
     /* effect: if Name is defined, print it, a colon, and a space */
{
  if (Name) {
    fprintf(stderr, Name);
    fprintf(stderr, ": ");
  }
}
static void warn_head(void)
     /* effect: if Name is defined, print it, and "Warning: " */
{
  print_name();
  fprintf(stderr, "Warning: ");
}
static void error_head(void)
     /* effect: if Name is defined, print it, and "ERROR: " */
{
  print_name();
  fprintf(stderr, "ERROR: ");
}
void usage()
     /* effect: print usage message on standard error output, and exit(1)
      */
{
  fprintf(stderr, "Usage: ");
  fprintf(stderr, Name);
  fprintf(stderr, " %s\n", Usage);
  exit(1);
}
/* For purposes of this homework, STOP HERE. 
   The functions below, warning, error, sys_err and sys_warning, 
   can't easily be fixed. 
   Instead of trying to fix them, we will just consider them to be 
   just as dangerous as a call to printf in any further analysis. 
*/

void warning(const char *fmt, ...)
     /* effect: print Name (if defined), "Warning: ",
	then a message controlled by fmt and the other args,
	then a newline on standard error output.
      */
{
  va_list args;

  va_start(args, fmt);
  warn_head();
  vfprintf(stderr, fmt, args);
  fprintf(stderr, "\n");
  va_end(args);
}

void error(const char *fmt, ...)
     /* effect: print Name (if defined), "Warning: ",
	then a message controlled by fmt and the other args,
	then a newline on standard error output.
      */
{
  va_list args;

  va_start(args, fmt);
  error_head();
  vfprintf(stderr, fmt, args);
  fprintf(stderr, "\n");
  va_end(args);

  exit(1);
}

void sys_err(const char *fmt, ...)
     /* requires: 0 <= errno && errno < sys_nerr */
     /* effect: print Name (if defined), "ERROR: ", msg, a colon, a space,
        the standard Unix system error message, and a newline
        on standard error output, then exit(1).
      */
{
  va_list args;

  va_start(args, fmt);
  error_head();
  vfprintf(stderr, fmt, args);
  if (0 <= errno && errno < sys_nerr) {
    fprintf(stderr, ": %s", sys_errlist[errno]);
  }
  fprintf(stderr, "\n");
  va_end(args);
  exit(1);
}

void sys_warning(const char *fmt, ...)
     /* requires: 0 <= errno && errno < sys_nerr */
     /* effect: print Name (if defined), "Warning: ", msg, a colon,
        a space, then the standard Unix system error message, and a newline
        on standard error output.
      */
{
  va_list args;

  va_start(args, fmt);
  warn_head();
  vfprintf(stderr, fmt, args);
  if (0 <= errno && errno < sys_nerr) {
    fprintf(stderr, ": %s", sys_errlist[errno]);
  }
  fprintf(stderr, "\n");
  va_end(args);
}
